#include <WiFi.h>
#include <HTTPClient.h>
#include <ArduinoJson.h>
#include "time.h"
#include <Wire.h>
#include <LiquidCrystal_I2C.h>
LiquidCrystal_I2C lcd(0x3F, 20, 4);           // у переходника дисплея адрес 0x3F
#include "iRusFont.h"
#include "Adafruit_SHT31.h"
Adafruit_SHT31 sht31 = Adafruit_SHT31();      // объект для работы с датчиком
#include <EEPROM.h>
#define EEPROM_SIZE 255

StaticJsonDocument<850> doc;                  // Выделить буфер для документа JSON
String jsonBuffer;
String S = "";                                // для всякой хренотени

enum StateR                                   // перечисления для правого энкодера
{
  DEVICE_OFF,                                 // радио включено
  DEVICE_ON,                                  // радио выключено
  DEVICE_TOTAL                                // всего состояний
};
uint8_t deviceState = DEVICE_OFF;             // значение по умолчанию выключено

enum StateL                                   // перечисления для правого энкодера
{
  MODE_NONE,                                  // неактивно
  MODE_RSETUP,                                // настройка радио
  MODE_TOTAL                                  // всего настроек
};
uint8_t radioMode = MODE_NONE;                // значение по умолчанию выключено


#include "func_wifi.h"
#include "func_weather.h"
#include "func_radio.h"
#include "buttons.h"
#include "encoderL.h"
#include "encoderR.h"

unsigned long previousMillis = 0;             // предыдущее значение времени
const long interval = 500;                    // интервал переключений
uint8_t sec05 = 0;                            // для отсчета полсекунды
uint8_t  minOWCount = 0;                      // для отсчета минут для запроса openweathermap
const uint8_t minOW = 10;                     // запрашивать погоду каждые 10 минут
uint16_t  minDTCount = 0;                     // для отсчета минут для синхронизации времени
const uint16_t minDT = 1440;                  // синхронизировать время каждые 24*60= 1440 минут (сутки)

void setup()
{
  Serial.begin(115200);
  EEPROM.begin(EEPROM_SIZE);                  // Инициализация памяти
  sht31.begin(0x44);                          // инициализация датчика темп/влажн
  lcd.init();                                 // инициализация дисплея
  lcd.backlight();                            // включаем подсветку
  rusClear();                                 // очистка экрана
  buttonsInit ();                             // инициализация кнопок
  encoderLinit();                             // инициализация левого энкодера
  encoderRinit();                             // инициализация правого энкодера
  radio.setup(RESET_PIN, 21);                 // инициализация радио модуля
  loadMemory ();                              // подгружаем станции из памяти

  wifi_connect ();                            // подключаемся к Wi-Fi

  configTime(gmtOffset_sec, daylightOffset_sec, ntpServer);  // Синхронизируем внутренние часы с временем из интернета
  printWeather ();                                           // просим погоду
}

void loop() {

  refreshButtobs ();                                         // Обновляем состояние кнопок энкодеров


  if (btRState == true) {                                  // Кнопка правого энкодера точно была нажата

    if (btRPressed == false) {                               // кнопка была отпущена после последнего нажатия


      if (radioMode == MODE_NONE) {                        // вкл/выкл

        deviceState = ++deviceState % DEVICE_TOTAL;
        switch (deviceState) {
          case DEVICE_OFF:
            radioOFF();
            break;
          case DEVICE_ON:
            radioON ();
            break;
        }

      } 

      if (radioMode == MODE_RSETUP) {                       // сохранение станции
        radioSetupOK ();                                    // процедура настройки радио ОК
        Serial.println('P');
      }

      

      btRPressed = true;                                     // запомнили что учли это нажатие
    }
  } else {                                                   // кнопка отпущена
    btRPressed = false;
  }




  if (btLState == true) {                                    // Кнопка левого энкодера точно была нажата

    if (btLPressed == false) {                               // кнопка была отпущена после последнего нажатия

      Serial.println('L');                                   // Обработка события нажата кнопка левого энкодера

      if (deviceState != DEVICE_OFF) {                       // Кнопка левого энкодера реагирует только во включеном состоянии

        radioMode = ++radioMode % MODE_TOTAL;

        switch (radioMode) {
          case MODE_NONE:                                   // выход из режима настройки

            showChFreq (memFrequency[channel]);             // обновляем дисплей
            showVolume ();                                  // обновляем дисплей

            break;

          case MODE_RSETUP:                                 // режим настройки радио

            rSetup = 0;
            radioSetup ();

            break;
        }

      } else {
        //заначка
      }

      btLPressed = true;                                     // запомнили что учли это нажатие
    }
  } else {                                                   // кнопка отпущена
    btLPressed = false;
  }


  if (encoderCountR != 0) {        // покрутили ручку правого энкодера
    if (encoderCountR > 0) encoderRightClock ();
    if (encoderCountR < 0) encoderRightNoClock () ;
    encoderCountR = 0;
    Serial.println(encoderCountR);
  }

  if (encoderCountL != 0) {       // покрутили ручку левого энкодера
    if (encoderCountL > 0) encoderLeftClock ();
    if (encoderCountL < 0) encoderLeftNoClock () ;
    encoderCountL = 0;
    Serial.println(encoderCountL);
  }


  unsigned long currentMillis = millis();

  if (currentMillis - previousMillis >= interval) {

    previousMillis = currentMillis;
    sec05++;

    if (sec05 & 1) iPrintString (":", 2, 0); else iPrintString (" ", 2, 0);   // раз в полсекунды мигаем точками

    if (sec05 & 1) {                                                          // раз в полсекунды обновляем время и температуру
      printTH ();
      printLocalTime();
    }

    // прошла одна минута
    if (sec05 >= 120) {
      sec05 = 0;
      minOWCount++;
      minDTCount++;

      //Прошло 10 минут
      if (minOWCount >= minOW) {
        minOWCount = 0;
        // печатаем погоду
        printWeather();
      }

      //Прошло 1440 минут (сутки)
      if (minDTCount >= minDT) {
        minDTCount = 0;
        //синхронизируем время
        configTime(gmtOffset_sec, daylightOffset_sec, ntpServer);
      }
    }
  } else delay(1); //вывод на дисплей импользуется в качестве задержки или просто задержка для антидребезга

}
