#include <SI470X.h>                  // библиотека для работы с радио модулем
#define RESET_PIN 14                 // пин сброса радиомодуля
SI470X radio;                        // объект для работы с радио
uint8_t radioVolume = 0;              // громкость
uint8_t channel = 0;                  // номер радиостанции
uint16_t memFrequency [99];           // массив с частотами станций

void showChFreq (float F) {                 // отображение номера канала и частоты
  iPrintString("                    ", 0, 2);
  uint8_t channelShow = channel + 1;
  lcd.setCursor (0, 2);                     // выводим номер канала
  if (channelShow < 10) {
    lcd.print(0);
    lcd.print(channelShow);
  } else lcd.print(channelShow);
  lcd.print(' ');
  F = F / 100;
  lcd.print(F, 1);
}

void loadMemory () {                 // загрузка списка станций
  EEPROM.get(0, memFrequency);

  // если это первое включение радио записываем во все ячейки памяти значение начала FM диапазона
  if (memFrequency [0] == 65535) {

    for (uint8_t i = 0; i <= 98; i++) {
      if (memFrequency [i] == 65535) memFrequency [i] = 8750; // начало диапазона
    }
    EEPROM.put(0, memFrequency);
    EEPROM.commit();
  }

}

void saveMemory () {                 // сохранение списка станций
  memFrequency[channel] = radio.getFrequency();
  EEPROM.put(0, memFrequency);
  EEPROM.commit();
}

void showVolume () {                  // отображение громкости
  iPrintString("VOLUME   ", 0, 3);
  iPrintString("VOLUME ", 0, 3);
  lcd.print(radio.getVolume());
}

uint8_t rSetup = 0;                     // выбор канала / частоты

void radioSetup () {                    // настройка радио

  rSetup++;
  switch (rSetup) {
    case 1:
      iPrintString("КАНАЛ?      ", 0, 3);
      break;
    case 2:
      iPrintString("ЧАСТОТА?    ", 0, 3);
      break;
    case 3:
      rSetup = 0;
      break;
  }

}

void radioSetupOK () {                  // процедура подверждения
  saveMemory ();
  iPrintString("СОХРАНЕНО   ", 0, 3);
  delay(1000);
  rSetup = 0;
  radioSetup ();
}

void radioON () {                    // Функция включаем радио
  radio.setFrequency(memFrequency[channel]);
  radio.setMute(false);
  radio.setVolume(radioVolume);
  iPrintString("                    ", 0, 2);
  iPrintString("                    ", 0, 3);
  uint8_t channelShow = channel + 1;
  lcd.setCursor (0, 2);              // выводим номер канала
  if (channelShow < 10) {
    lcd.print(0);
    lcd.print(channelShow);
  } else lcd.print(channelShow);
  lcd.print(' ');
  float F = memFrequency[channel];
  F = F / 100;
  lcd.print(F, 1);
  showVolume ();
  //lcd.print(radioVolume);
}

void radioOFF () {
  radio.setMute(true);
  rusClear();                      // очистка экрана
  printWeather ();                 // заполняем экран погодой
}

void encoderRightClock () {            // правый экнодер по часовой стрелке

  if (deviceState == DEVICE_ON) {      // правый энкодер когда радио включено

    switch (radioMode) {
      case 0:                          // регулировка звука +
        radio.setVolumeUp();
        radioVolume = radio.getVolume();  //сохраняем для кнопки включения
        showVolume ();
        break;

      case 1:                          // режим настройки радио

        switch (rSetup) {
          case 1:
            if (channel < 98) channel++;
            radio.setFrequency(memFrequency[channel]);
            showChFreq(memFrequency[channel]);
            break;
          case 2:
            radio.setFrequencyUp();
            showChFreq(radio.getFrequency());
            break;
        }

        break;
    }

  } else {                             // правый энкодер когда радио выключено
    // заначка
  }

}

void encoderRightNoClock () {          // правый экнодер против часовой стрелке

  if (deviceState == DEVICE_ON) {      // правый энкодер когда радио включено

    switch (radioMode) {
      case 0:                          // регулировка звука -
        radio.setVolumeDown();
        radioVolume = radio.getVolume(); //сохраняем для кнопки включения
        showVolume ();
        break;

      case 1:                          // режим настройки радио

        switch (rSetup) {
          case 1:                     // выбор канала
            if (channel > 0) channel--;
            radio.setFrequency(memFrequency[channel]);
            showChFreq(memFrequency[channel]);
            break;
          case 2:
            radio.setFrequencyDown();
            showChFreq(radio.getFrequency());
            break;
        }

        break;
    }


  } else {                             // правый энкодер когда радио выключено
    // заначка
  }

}

void encoderLeftClock () {             // левый энкодер по часовой стрелки

  if (deviceState == DEVICE_ON) {      // левый энкодер когда радио включено

    switch (radioMode) {
      case 0:                          // канал +
        if (channel < 98) channel++;
        showChFreq(memFrequency[channel]);
        radio.setFrequency(memFrequency[channel]);
        break;
      case 1:                          // режим настройки радио
        radioSetup ();
        break;
    }

  } else {                             // левый энкодер когда радио выключено
    // заначка
  }

}

void encoderLeftNoClock () {           // левый энкодер против часовой стрелки

  if (deviceState == DEVICE_ON) {      // левый энкодер когда радио включено

    switch (radioMode) {
      case 0:                          // канал -
        if (channel > 0) channel--;
        showChFreq(memFrequency[channel]);
        radio.setFrequency(memFrequency[channel]);
        break;
      case 1:                          // режим настройки радио
        radioSetup ();
        break;
    }

  } else {                             // левый  энкодер когда радио выключено
    // заначка
  }

}
